#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the schema helper."""

import unittest

from acstore.helpers import schema

from tests import test_lib as shared_test_lib


class SchemaHelperTest(shared_test_lib.BaseTestCase):
  """Tests for the schema helper."""

  # pylint: disable=protected-access

  def testHasDataType(self):
    """Tests the HasDataType function."""
    result = schema.SchemaHelper.HasDataType('str')
    self.assertTrue(result)

    result = schema.SchemaHelper.HasDataType('test')
    self.assertFalse(result)

  def testRegisterDataType(self):
    """Tests the RegisterDataType function."""
    number_of_data_types = len(schema.SchemaHelper._data_types)

    schema.SchemaHelper.RegisterDataType('test', {'json': None})

    try:
      self.assertEqual(
          len(schema.SchemaHelper._data_types), number_of_data_types + 1)

      with self.assertRaises(KeyError):
        schema.SchemaHelper.RegisterDataType('test', {'json': None})

    finally:
      schema.SchemaHelper.DeregisterDataType('test')

    self.assertEqual(
        len(schema.SchemaHelper._data_types), number_of_data_types)

  def testRegisterDataTypes(self):
    """Tests the RegisterDataTypes function."""
    number_of_data_types = len(schema.SchemaHelper._data_types)

    schema.SchemaHelper.RegisterDataTypes({'test': {'json': None}})

    try:
      self.assertEqual(
          len(schema.SchemaHelper._data_types), number_of_data_types + 1)

      with self.assertRaises(KeyError):
        schema.SchemaHelper.RegisterDataTypes({'test': {'json': None}})

    finally:
      schema.SchemaHelper.DeregisterDataType('test')

    self.assertEqual(
        len(schema.SchemaHelper._data_types), number_of_data_types)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the YAML-based attribute container definitions file."""

import io
import unittest

from acstore import errors
from acstore.helpers import yaml_definitions_file

from tests import test_lib as shared_test_lib


class YAMLAttributeContainerDefinitionsFileTest(shared_test_lib.BaseTestCase):
  """Tests for the YAML-based attribute container definitions file."""

  # pylint: disable=protected-access

  _FORMATTERS_YAML = {
      'name': 'windows_eventlog_message_file',
      'attributes': [
          {'name': 'path', 'type': 'str'},
          {'name': 'windows_path', 'type': 'str'}]}

  def testReadDefinition(self):
    """Tests the _ReadDefinition function."""
    test_definitions_file = (
        yaml_definitions_file.YAMLAttributeContainerDefinitionsFile())

    container_class = test_definitions_file._ReadDefinition(
        self._FORMATTERS_YAML)

    self.assertIsNotNone(container_class)
    self.assertEqual(
        container_class.CONTAINER_TYPE, 'windows_eventlog_message_file')
    self.assertEqual(
        container_class.SCHEMA, {'path': 'str', 'windows_path': 'str'})

    with self.assertRaises(errors.ParseError):
      test_definitions_file._ReadDefinition({})

    with self.assertRaises(errors.ParseError):
      test_definitions_file._ReadDefinition({
          'name': 'windows_eventlog_message_file',
          'attributes': []})

    with self.assertRaises(errors.ParseError):
      test_definitions_file._ReadDefinition({
          'name': 'windows_eventlog_message_file',
          'attributes': [{'type': 'str'}]})

    with self.assertRaises(errors.ParseError):
      test_definitions_file._ReadDefinition({
          'name': 'windows_eventlog_message_file',
          'attributes': [{'name': 'path'}]})

    with self.assertRaises(errors.ParseError):
      test_definitions_file._ReadDefinition({
          'name': 'windows_eventlog_message_file',
          'attributes': [{'name': 'path', 'type': 'bogus'}]})

  def testReadFromFileObject(self):
    """Tests the _ReadFromFileObject function."""
    test_file_path = self._GetTestFilePath(['definitions.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    test_definitions_file = (
        yaml_definitions_file.YAMLAttributeContainerDefinitionsFile())

    with io.open(test_file_path, 'r', encoding='utf-8') as file_object:
      definitions = list(test_definitions_file._ReadFromFileObject(file_object))

    self.assertEqual(len(definitions), 1)

  def testReadFromFile(self):
    """Tests the ReadFromFile function."""
    test_file_path = self._GetTestFilePath(['definitions.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    test_definitions_file = (
        yaml_definitions_file.YAMLAttributeContainerDefinitionsFile())

    definitions = list(test_definitions_file.ReadFromFile(test_file_path))

    self.assertEqual(len(definitions), 1)

    self.assertEqual(
        definitions[0].CONTAINER_TYPE, 'windows_eventlog_message_file')


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""YAML-based attribute container definitions file."""

import yaml

from acstore import errors
from acstore.containers import interface
from acstore.helpers import schema


# TODO: merge this into interface.AttributeContainer once Plaso has been
# changed to no longer support attributes containers without a schema.
class AttributeContainerWithSchema(interface.AttributeContainer):
  """Attribute container with schema."""

  SCHEMA = {}


class YAMLAttributeContainerDefinitionsFile(object):
  """YAML-based attribute container definitions file.

  A YAML-based attribute container definitions file contains one or more
  attribute container definitions. An attribute container definition consists
  of:

  name: 'windows_eventlog_message_file'
  attributes:
  - name: path
    type: str
  - name: windows_path
    type: str

  Where:
  * name, unique identifier of the attribute container;
  * attributes, defines the attributes of the container.
  """

  _SUPPORTED_DATA_TYPES = frozenset([
      'AttributeContainerIdentifier',
      'bool',
      'int',
      'str',
      'timestamp'])

  _SUPPORTED_KEYS = frozenset([
      'name',
      'attributes'])

  def _ReadDefinition(self, definition_values):
    """Reads a definition from a dictionary.

    Args:
      definition_values (dict[str, object]): attribute container definition
          values.

    Returns:
      AttributeContainer: an attribute container.

    Raises:
      ParseError: if the definition is not set or incorrect.
    """
    if not definition_values:
      raise errors.ParseError('Missing attribute container definition values.')

    different_keys = set(definition_values) - self._SUPPORTED_KEYS
    if different_keys:
      different_keys = ', '.join(different_keys)
      raise errors.ParseError(f'Undefined keys: {different_keys:s}')

    container_name = definition_values.get('name', None)
    if not container_name:
      raise errors.ParseError(
          'Invalid attribute container definition missing name.')

    attributes = definition_values.get('attributes', None)
    if not attributes:
      raise errors.ParseError((
          f'Invalid attribute container definition: {container_name:s} '
          f'missing attributes.'))

    class_name = ''.join([
        element.title() for element in container_name.split('_')])

    class_attributes = {'CONTAINER_TYPE': container_name}
    container_schema = {}

    for attribute_index, attribute_values in enumerate(attributes):
      attribute_name = attribute_values.get('name', None)
      if not attribute_name:
        raise errors.ParseError((
            f'Invalid attribute container definition: {container_name:s} name '
            f'missing of attribute: {attribute_index:d}.'))

      if attribute_name in class_attributes:
        raise errors.ParseError((
            f'Invalid attribute container definition: {container_name:s} '
            f'attribute: {attribute_name:s} already set.'))

      attribute_data_type = attribute_values.get('type', None)
      if not attribute_data_type:
        raise errors.ParseError((
            f'Invalid attribute container definition: {container_name:s} type '
            f'missing of attribute: {attribute_name:s}.'))

      if not schema.SchemaHelper.HasDataType(attribute_data_type):
        raise errors.ParseError((
            f'Invalid attribute container definition: {container_name:s} type '
            f'attribute: {attribute_name:s} unsupported data type: '
            f'{attribute_data_type:s}.'))

      class_attributes[attribute_name] = None
      container_schema[attribute_name] = attribute_data_type

    class_attributes['SCHEMA'] = container_schema

    # TODO: add support for _SERIALIZABLE_PROTECTED_ATTRIBUTES.

    return type(class_name, (AttributeContainerWithSchema, ), class_attributes)

  def _ReadFromFileObject(self, file_object):
    """Reads the definitions from a file-like object.

    Args:
      file_object (file): definitions file-like object.

    Yields:
      AttributeContainer: an attribute container.
    """
    yaml_generator = yaml.safe_load_all(file_object)

    for yaml_definition in yaml_generator:
      yield self._ReadDefinition(yaml_definition)

  def ReadFromFile(self, path):
    """Reads the definitions from a YAML file.

    Args:
      path (str): path to a definitions file.

    Yields:
      AttributeContainer: an attribute container.
    """
    with open(path, 'r', encoding='utf-8') as file_object:
      for yaml_definition in self._ReadFromFileObject(file_object):
        yield yaml_definition
